/*
 * fullfullscreen.js
 *
 * Full Fullscreen Firefox extension
 * emsearcy@gmail.com
 *
 * Licensed under MPL (http://www.mozilla.org/MPL/MPL-1.1.html)
 */

var fullfullscreen = {
    prefs_ffs: null,
    prefs_b_tabs: null,
    prefs_b_fs: null,
    notabs: false,
    noautohide: false,

    // initialize the extension
    startup: function()
    {
        // grab services
        prefprovider = '@mozilla.org/preferences-service;1';
        this.prefs_ffs = Components.classes[prefprovider]
            .getService(Components.interfaces.nsIPrefService)
            .getBranch('fullfullscreen.');
        this.prefs_ffs.QueryInterface(Components.interfaces.nsIPrefBranch2);
        this.prefs_ffs.addObserver('', this, false);

        this.prefs_b_tabs = Components.classes[prefprovider]
            .getService(Components.interfaces.nsIPrefService)
            .getBranch('browser.tabs.');

        this.prefs_b_fs = Components.classes[prefprovider]
            .getService(Components.interfaces.nsIPrefService)
            .getBranch('browser.fullscreen.');

        // fullscreen style
        this.notabs = this.prefs_ffs.getBoolPref('notabs');
        this.noautohide = this.prefs_ffs.getBoolPref('noautohide');
        this.prefs_b_fs.setBoolPref('autohide', !this.noautohide);
        this.autohidestyle();

        // add listener for new tabs
        gBrowser.tabContainer.addEventListener('TabOpen', function(e) {
                fullfullscreen.checknewtab();
            }, false);

        // initial fullscreen (has to have delay)
        if (this.prefs_ffs.getBoolPref('start')) {
            setTimeout(fullfullscreen.fullscreen, 500);
        }

        return true;
    },

    // handler for prefs change
    observe: function(subject, topic, data)
    {
        if (topic != 'nsPref:changed') return;

        switch(data) {
            case 'noautohide':
                // noautohide setting was changed
                this.noautohide = this.prefs_ffs.getBoolPref('noautohide');
                this.prefs_b_fs.setBoolPref('autohide', !this.noautohide);
                this.autohidestyle();
                break;

            case 'notabs':
                // notabs setting was changed
                this.notabs = this.prefs_ffs.getBoolPref('notabs');
                if (!this.notabs && !gBrowser.getStripVisibility() &&
                        (gBrowser.tabContainer.childNodes.length > 1 ||
                        !this.prefs_b_tabs.getBoolPref('autoHide'))) {
                    // notabs is now false, the tab bar is hidden, and we
                    // either have more than one tab or "Always show tab bar"
                    // is true
                    gBrowser.setStripVisibilityTo(true);
                } else if (this.notabs && this.noautohide && window.fullScreen &&
                        gBrowser.getStripVisibility()) {
                    // notabs is now true, autohiding is disabled, we're
                    // currently fullscreen, and the tab bar is visible
                    gBrowser.setStripVisibilityTo(false);
                }
                break;
        }
    },

    // handle changes to/from autohide
    autohidestyle: function()
    {
        var node = null;

        if (window.fullScreen && this.noautohide) {
            // exit fullscreen
            BrowserFullScreen();
            // remove fullscreen protection
            node = document.getElementById('nav-bar');
            if (node) node.setAttribute('fullscreentoolbar', 'false');
            // enter fullscreen
            this.fullscreen();
        } else if (window.fullScreen) {
            // exit fullscreen
            BrowserFullScreen();
            // show tab bar if more than one tab or "Always show tab bar" true
            if (gBrowser.tabContainer.childNodes.length > 1 ||
                    !this.prefs_b_tabs.getBoolPref('autoHide')) {
                gBrowser.setStripVisibilityTo(true);
            }
            // restore fullscreen protection
            node = document.getElementById('nav-bar');
            if (node) node.setAttribute('fullscreentoolbar', 'true');
            // enter fullscreen
            this.fullscreen();
        } else if (this.noautohide) {
            // remove fullscreen protection
            node = document.getElementById('nav-bar');
            if (node) node.setAttribute('fullscreentoolbar', 'false');
        } else {
            // restore fullscreen protection
            node = document.getElementById('nav-bar');
            if (node) node.setAttribute('fullscreentoolbar', 'true');
        }
    },

    // check which way fullscreen changing
    checktransition: function()
    {
        // window.fullScreen is value *before* transitioning
        if (this.noautohide && this.notabs && !window.fullScreen &&
                gBrowser.getStripVisibility()) {
            // hide tab bar
            gBrowser.setStripVisibilityTo(false);
        } else if (window.fullScreen && !gBrowser.getStripVisibility() &&
                (gBrowser.tabContainer.childNodes.length > 1 ||
                !this.prefs_b_tabs.getBoolPref('autoHide'))) {
            // show tab bar
            gBrowser.setStripVisibilityTo(true);
        }
    },

    checknewtab: function()
    {
        if (this.noautohide && this.notabs && window.fullScreen &&
                gBrowser.getStripVisibility()) {
            // hide tab bar
            gBrowser.setStripVisibilityTo(false);
        }
    },

    // workaround bug 196714
    fullscreen: function()
    {
        if (window && window.screenX == 0 && window.screenY == 0 &&
                window.outerWidth == screen.width &&
                window.outerHeight == screen.height) {
            window.resizeTo(screen.width - 1, screen.height);
        }
        BrowserFullScreen();
    },

    // clean up
    shutdown: function()
    {
        this.prefs_ffs.removeObserver("", this);
    }
}

window.addEventListener('load', function(e) {
        fullfullscreen.startup();
    }, false);
window.addEventListener('unload', function(e) {
        fullfullscreen.shutdown();
    }, false);
window.addEventListener('fullscreen', function(e) {
        fullfullscreen.checktransition();
    }, false);
